package ru.dbotthepony.kstarbound.world.api

import java.io.DataInputStream

data class MutableCell(
	override val foreground: MutableTileState = MutableTileState(),
	override val background: MutableTileState = MutableTileState(),
	override val liquid: MutableLiquidState = MutableLiquidState(),

	override var dungeonId: Int = 0,
	override var blockBiome: Int = 0,
	override var envBiome: Int = 0,
	override var isIndestructible: Boolean = false,
) : AbstractCell() {
	fun read(stream: DataInputStream): MutableCell {
		foreground.read(stream)
		background.read(stream)
		liquid.read(stream)

		stream.skipNBytes(1) // collisionMap

		dungeonId = stream.readUnsignedShort()
		blockBiome = stream.readUnsignedByte()
		envBiome = stream.readUnsignedByte()
		isIndestructible = stream.readBoolean()

		stream.skipNBytes(1) // unknown
		return this
	}

	override fun tile(background: Boolean): MutableTileState {
		if (background)
			return this.background
		else
			return this.foreground
	}

	override fun immutable(): ImmutableCell {
		return POOL.intern(ImmutableCell(foreground.immutable(), background.immutable(), liquid.immutable(), dungeonId, blockBiome, envBiome, isIndestructible))
	}

	override fun mutable(): MutableCell {
		return this
	}
}
